@@ -5,6 +5,7 @@ except ImportError:
     print('This feature requires pandas, please install with `pip install pandas`')
     raise
 from pyrfuniverse.envs.base_env import RFUniverseBaseEnv
+import pyrfuniverse.attributes as attr
 
 mesh_path = '../Mesh/drink1/drink1.obj'
 pose_path = '../Mesh/drink1/grasps_rfu.csv'
@@ -15,19 +16,8 @@ positions = data[:, 0:3].reshape(-1).tolist()
 quaternions = data[:, 3:7].reshape(-1).tolist()
 
 env = RFUniverseBaseEnv()
+grasp_sim = env.InstanceObject(id=123123, name='GraspSim', attr_type=attr.GraspSimAttr)
+grasp_sim.ShowGraspPose(mesh=os.path.abspath(mesh_path), gripper='SimpleFrankaGripper', positions=positions, quaternions=quaternions)
 
-env.asset_channel.set_action(
-    'InstanceObject',
-    id=123123,
-    name='GraspSim'
-)
-env.instance_channel.set_action(
-    'ShowGraspPose',
-    id=123123,
-    mesh=os.path.abspath(mesh_path),
-    gripper='SimpleFrankaGripper',
-    positions=positions,
-    quaternions=quaternions,
-)
-while True:
+while 1:
     env._step()
